{# 
output_file: README.md
description: Markdown file with challenges grouped by categories
prettify: true
#}
# Challenge Index

{% set grouped = {} %}
{% for c in challenges %}{% set _ = grouped.setdefault(c.data.category, []).append(c) %}{% endfor %}

{% for category, items in grouped.items() %}
## {{ category }}
| Name | Points | Solved | Path |
|------|--------|--------|------|
{% for c in items %}
| {{ c.data.name }} | {{ c.data.value }} | {{ "✅" if c.data.solved else "❌" }} | [Link]({{ c.path }}) |
{% endfor %}
{% endfor %}